﻿using System;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;

/// <summary>
/// Перечисление с возможными уровнями уверенности, что это паук
/// </summary>

public enum SpiderMatch
{
  NoMatch,     // ни пользовательский агент, ни IP не совпадают с пауком
  MatchUA,     // пользовательский агент совпадает с пауком
  MatchIP,     // IP совпадает с пауком
  MatchUAandIP // и пользовательский агент, и IP совпадают с пауком
}

/// <summary>
/// Cloak — функции поддержки маскирования
/// </summary>

public static class Cloak
{
  // возвращает уровень уверенности, что данный посетитель является пауком
  // (0 = нет совпадения, 2 = совпадение пользовательского агента,
  // 3 = совпадение ip, 5 = совпадение п/а и ip)
  public static SpiderMatch IsSpider()
  {
    return IsSpider("");
  }

  public static SpiderMatch IsSpider(string spiderName)
  {
    // по умолчанию уровень уверенности = 0
    SpiderMatch confidence = 0;

    // пользовательский агент посетителя
    string visitorUA = HttpContext.Current.Request["HTTP_USER_AGENT"];
    string visitorIP = HttpContext.Current.Request["REMOTE_ADDR"];

    // проверка совпадения пользовательского агента и IP 
    bool uaMatches = CloakDb.CheckUA(visitorUA, spiderName);
    bool ipMatches = CloakDb.CheckIP(visitorIP, spiderName);

    // получение уровня уверенности
    if (uaMatches && ipMatches) confidence = SpiderMatch.MatchUAandIP;
    else if (uaMatches) confidence = SpiderMatch.MatchUA;
    else if (ipMatches) confidence = SpiderMatch.MatchIP;
    else confidence = SpiderMatch.NoMatch;

    // возврат уровня уверенности, что посетитель является пауком
    return confidence;
  }

  // проверка, что данный IP принадлежит пауку
  public static bool IsSpiderIP(string ip)
  {
    return CloakDb.CheckIP(ip, "");
  }

  // проверка, что данный IP принадлежит указанному пауку
  public static bool IsSpiderIP(string ip, string spiderName)
  {
    return CloakDb.CheckIP(ip, spiderName);
  }

  // проверка, что данный пользовательский агент принадлежит пауку
  public static bool IsSpiderUA(string ua)
  {
    return CloakDb.CheckUA(ua, "");
  }

  // проверка, что данный пользовательский агент принадлежит указанному пауку
  public static bool IsSpiderUA(string ua, string spiderName)
  {
    return CloakDb.CheckUA(ua, spiderName);
  }

  // обновление данных маскирования; возвращает true, если обновление выполнено
  public static bool UpdateAll()
  {
    // чтение версии и даты обновления из базы данных
    string dbVersion = "";
    DateTime dbUpdatedOn = new DateTime();
    CloakDb.GetLastUpdate(ref dbVersion, ref dbUpdatedOn);

    // выход, если база обновлена не позже чем неделю назад
    if (DateTime.Now - new TimeSpan(7, 0, 0, 0) < dbUpdatedOn) return false;

    // чтение самой свежей версии, опубликованной iplists
    WebRequest request = HttpWebRequest.Create(
      "http://www.iplists.com/nw/version.php");
    Stream responseStream = request.GetResponse().GetResponseStream();
    StreamReader reader = new StreamReader(responseStream);
    string latestVersion = reader.ReadLine();

    // выход, если самая свежая версия базы уже имеется
    if (dbVersion == latestVersion) return false;

    // обновление версии и даты базы данных
    CloakDb.SetLastUpdate(latestVersion, DateTime.Now);

    // обновление базы данных
    updateSpiderData("google", "http://www.iplists.com/nw/google.txt");
    updateSpiderData("yahoo", "http://www.iplists.com/nw/inktomi.txt");
    updateSpiderData("msn", "http://www.iplists.com/nw/msn.txt");
    updateSpiderData("ask", "http://www.iplists.com/nw/askjeeves.txt");
    updateSpiderData("altavista", "http://www.iplists.com/nw/altavista.txt");
    updateSpiderData("lycos", "http://www.iplists.com/nw/lycos.txt");
    updateSpiderData("wisenut", "http://www.iplists.com/nw/wisenut.txt");

    // возврат true, что означает, что данные обновлены
    return true;
  }

  private static void updateSpiderData(string spiderName, string spiderUrl)
  {
    // создание объекта запроса
    WebRequest request = HttpWebRequest.Create(spiderUrl);

    // создание потока для чтения данных из URL
    Stream responseStream = request.GetResponse().GetResponseStream();

    // загрузка списка IP-адресов в строку
    StreamReader reader = new StreamReader(responseStream);
    string inputString = reader.ReadToEnd();
    reader.Close();

    // удаление данных из базы для паука
    CloakDb.DeleteSpider(spiderName);

    // выборка пользовательских агентов с помощью регулярного выражения
    MatchCollection matches;
    matches = Regex.Matches(inputString,
      @"^# UA ""(?<url>.*)""$",
      RegexOptions.ExplicitCapture | RegexOptions.Multiline);

    // добавление в базу каждого обнаруженного пользовательского агента
    foreach (Match m in matches)
    {
      string ua = m.Groups["url"].ToString();
      CloakDb.AddSpiderRecord(spiderName, "UA", ua);
    }

    // регулярное выражение для выборки IP-адресов паука
    matches = Regex.Matches(inputString,
      "^(?<ip>[0-9.]+)$", RegexOptions.Multiline);

    // добавление в базу данных совпавших IP-адресов
    foreach (Match m in matches)
    {
      string ip = m.Groups["ip"].ToString();
      CloakDb.AddSpiderRecord(spiderName, "IP", ip);
    }
  }
}