﻿using System;
using System.Web;
using System.Data;
using System.Configuration;

/// <summary>
/// Класс поддержки для работы с URL и их перенаправления
/// </summary>

public static class UrlTools
{
  // получение домена сайта из конфигурационного файла
  public readonly static string SiteDomain =
    ConfigurationManager.AppSettings["SiteDomain"];

  /* проверяет, что текущая страница загружена по ее стандартному URL;
   * выполняет перенаправление 301 на стандартный URL, если это не так */
  public static void CheckUrl()
  {
    HttpContext context = HttpContext.Current;
    HttpRequest request = HttpContext.Current.Request;

    // выборка параметров из строки запроса
    string productId = request.QueryString["ProductID"];
    string categoryId = request.QueryString["CategoryID"];
    string page = request.QueryString["Page"];

    // проверка URL-адресов категории-продукта
    if (categoryId != null && productId != null)
    {
      CheckCategoryProductUrl(categoryId, productId);
    }
    // проверка URL-адресов категории
    else if (categoryId != null && page != null)
    {
      CheckCategoryUrl(categoryId, page);
    }
  }

  // проверка URL категории на совместимость
  // при необходимости перенаправление 301 на нужный URL, или возврат кода 404
  public static void CheckCategoryUrl(string categoryId, string page)
  {
    // текущий HttpContext
    HttpContext context = HttpContext.Current;

    // URL, запрошенный посетителем
    string requestedUrl = context.Request.ServerVariables["HTTP_X_REWRITE_URL"];

    // выборка данных категории
    DataRow categoryRow = CatalogDb.GetCategory(categoryId);

    // при отсутствии категории или продукта в базе данных - возврат кода 404
    if (categoryRow == null)
    {
      NotFound();
    }

    // получение названия категории
    string categoryName = categoryRow["Name"].ToString();

    // получение стандартной версии URL
    string standardUrl = LinkFactory.MakeCategoryUrl(categoryName, categoryId, int.Parse(page));

    // при необходимости перенаправление 301 на правильный URL 
    if (SiteDomain + requestedUrl != standardUrl)
    {
      context.Response.Status = "301 Moved Permanently";
      context.Response.AddHeader("Location", standardUrl);
    }
  }

  // проверка совместимости URL категории-продукта
  // при необходимости перенаправление 301 на нужный URL или возврат кода 404 
  public static void CheckCategoryProductUrl(string categoryId, string productId)
  {
    // текущий HttpContext
    HttpContext context = HttpContext.Current;

    // URL, запрошенный посетителем
    string requestedUrl = context.Request.ServerVariables["HTTP_X_REWRITE_URL"];

    // выборка данных продукта и категории
    DataRow categoryRow = CatalogDb.GetCategory(categoryId);
    DataRow productRow = CatalogDb.GetProduct(productId);

    // при отсутствии категории или продукта в базе данных - возврат кода 404
    if (categoryRow == null || productRow == null)
    {
      NotFound();
    }

    // получение названий продукта и категории
    string categoryName = categoryRow["Name"].ToString();
    string productName = productRow["Name"].ToString();

    // получение стандартной версии URL
    string standardUrl = LinkFactory.MakeCategoryProductUrl(categoryName,
      categoryId, productName, productId);

    // при необходимости перенаправление 301 на правильный URL 
    if (SiteDomain + requestedUrl != standardUrl)
    {
      context.Response.Status = "301 Moved Permanently";
      context.Response.AddHeader("Location", standardUrl);
    }
  }

  // Загрузка страницы 404
  public static void NotFound()
  {
    HttpContext.Current.Server.Transfer("~/NotFound.aspx");
  }
}